/*******************************************************************************
 * Copyright (C) 2009
 * by 52 North Initiative for Geospatial Open Source Software GmbH
 * 
 * Contact: Andreas Wytzisk
 * 52 North Initiative for Geospatial Open Source Software GmbH
 * Martin-Luther-King-Weg 24
 * 48155 Muenster, Germany
 * info@52north.org
 * 
 * This program is free software; you can redistribute and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program (see gnu-gpl v2.txt). If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA or
 * visit the Free Software Foundation web page, http://www.fsf.org.
 ******************************************************************************/

package org.n52.security.authentication;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.security.Key;
import java.security.Principal;
import java.security.cert.Certificate;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;
import java.util.Set;

import javax.security.auth.Subject;

import org.apache.log4j.Logger;
import org.n52.security.authentication.principals.AttributePrincipal;
import org.n52.security.authentication.principals.NamedScope;
import org.n52.security.authentication.principals.RolePrincipal;
import org.n52.security.authentication.principals.Scope;
import org.n52.security.authentication.principals.UsernameIDPrincipal;
import org.opensaml.SAMLAssertion;
import org.opensaml.SAMLAttribute;
import org.opensaml.SAMLAttributeStatement;
import org.opensaml.SAMLAuthenticationStatement;
import org.opensaml.SAMLException;
import org.opensaml.SAMLNameIdentifier;
import org.opensaml.SAMLResponse;
import org.opensaml.SAMLStatement;
import org.opensaml.SAMLSubject;
import org.w3c.dom.Document;

import sun.security.krb5.internal.AuthContext;

/**
 * The SAMLTicket class basically provides a SAMLResponse <-> javax.security.auth.Subject mapping.
 */
public class SAMLTicket {

    private static Logger sLogger = Logger.getLogger(SAMLTicket.class);

    /**
     *
     */
    protected SAMLResponse m_samlResponse;

    /**
     * @param samlResponse
     *            String representation of an "UTF-8" coded SAMLTicket.
     * @throws SAMLException
     */
    public SAMLTicket(final String samlResponse) throws SAMLException {
        byte[] lSAMLReponseUTF8;
        try {
            lSAMLReponseUTF8 = samlResponse.getBytes("UTF-8");
        } catch (UnsupportedEncodingException e) {
            sLogger.error("Error while creating SAMLTicket from String", e);
            throw new SAMLException("Error while creating SAMLTicket from String", e);
        }
        m_samlResponse = new SAMLResponse(new ByteArrayInputStream(lSAMLReponseUTF8));
    }

    /**
     * @param pSAMLResponse
     * @throws SAMLException
     */
    public SAMLTicket(Document pSAMLResponse) throws SAMLException {
        this(new SAMLResponse(pSAMLResponse.getDocumentElement()));
    }

    /**
     * @param pSAMLResponse
     * @throws SAMLException
     */
    public SAMLTicket(SAMLResponse pSAMLResponse) throws SAMLException {
        m_samlResponse = pSAMLResponse;
    }

    /**
     * Verifies the signature of this SAML document.
     * 
     * @throws SAMLException
     *             if the signature could not be verified
     */
    public void verify() throws SAMLException {
        m_samlResponse.verify();
    }

    public void verify(Certificate pWASCert) throws SAMLException {
        m_samlResponse.verify(pWASCert);
    }

    /**
     * check if any assertion is expired, without any respect to clock differences.
     * 
     * @return if ticket is expired.
     */
    public boolean isExpired() {
        return isExpired(0);
    }

    /**
     * check if any assertion is expired, in relation to an allowed delta.
     * 
     * @param allowedDelta
     *            a delta in msec.
     * @return if ticket is expired.
     */
    public boolean isExpired(final long allowedDelta) {
        final long lNow = System.currentTimeMillis();
        Iterator lAssertIter = m_samlResponse.getAssertions();
        while (lAssertIter.hasNext()) {
            SAMLAssertion tAssert = (SAMLAssertion) lAssertIter.next();
            final Date notBefore = tAssert.getNotBefore();
            if (notBefore != null && (notBefore.getTime() - allowedDelta) > lNow) {
                return true;
            }
            final Date notOnOrAfter = tAssert.getNotOnOrAfter();
            if (notOnOrAfter != null && (notOnOrAfter.getTime() + allowedDelta) <= lNow) {
                return true;
            }
        }
        return false;
    }

    /*
     * public SAMLAssertion getInternalAssertion(){ Iterator it = mSAMLResponse.getAssertions(); if (it.hasNext())
     * return (SAMLAssertion) it.next(); return null; }
     */

    /**
     * @return
     */
    public Subject asSubject() {
        Subject lSubject = new Subject();
        updateSubject(lSubject);
        return lSubject;
    }

    /**
     * @return
     */
    public String asString() {
        return m_samlResponse.toString();
    }

    /**
     * @return
     */
    public String asBase64String() {
        try {
            return new String(m_samlResponse.toBase64());
        } catch (IOException e) {
            sLogger.error(e);
            return null;
        } catch (SAMLException e) {
            sLogger.error(e);
            return null;
        }
    }

    /**
     * @param pSubject
     */
    public void updateSubject(Subject pSubject) {
        if (pSubject == null) {
            throw new IllegalArgumentException("Subject must not be null.");
        }
        pSubject.getPrincipals().addAll(retrievePrincipals());
    }

    /**
     * @return
     */
    private Set retrievePrincipals() {
        HashSet lPrincipals = new HashSet(8);

        // TODO: check if response timed out

        SAMLAssertion samlFirstAssertion = (SAMLAssertion) m_samlResponse.getAssertions().next();

        // Iterate through statements and create appropriate principals
        Iterator itrSamlStatements = samlFirstAssertion.getStatements();
        while (itrSamlStatements.hasNext()) {
            SAMLStatement tSamlStatement = (SAMLStatement) itrSamlStatements.next();

            // evaluate AuthenticationStatement
            if (tSamlStatement instanceof SAMLAuthenticationStatement) {
                SAMLAuthenticationStatement samlAuthnStatement = (SAMLAuthenticationStatement) tSamlStatement;
                SAMLSubject samlSubject = samlAuthnStatement.getSubject();
                String userName = samlSubject.getNameIdentifier().getName();
                String userNameQualifier = samlSubject.getNameIdentifier().getNameQualifier();
                lPrincipals.add(new UsernameIDPrincipal(userName, userNameQualifier == null ? Scope.GLOBAL
                        : new NamedScope(userNameQualifier)));
            }

            // evaluate AttributeStatements
            if (tSamlStatement instanceof SAMLAttributeStatement) {
                SAMLAttributeStatement samlAttrStatement = (SAMLAttributeStatement) tSamlStatement;

                Iterator itrSamlAttributes = samlAttrStatement.getAttributes();
                while (itrSamlAttributes.hasNext()) {
                    SAMLAttribute tAtt = (SAMLAttribute) itrSamlAttributes.next();
                    Iterator itrSamlAttrValues = tAtt.getValues();
                    while (itrSamlAttrValues.hasNext()) {
                        String tValue = (String) itrSamlAttrValues.next();
                        lPrincipals.add(new AttributePrincipal(tAtt.getName(), tValue));
                    }
                }
            }
        }
        return lPrincipals;

    }

    /**
     * Subject that contains the relevant information for the SAMLResponse. Must contain exactly one UsernameIDPrincipal
     * which is used for
     * 
     * @return
     */

    /**
     * Creates a SAML ticket, i.e. a SAMLResponse, based on a set of {@link Principal} instances that are part of a
     * {@link Subject} stored in an {@link AuthenticationContext} instance.
     * 
     * 
     * @param authContext
     *            context of an authentication process that took place in the past. The {@link Subject} provided by
     *            {@link AuthenticationContext#getSubject()} must contain exactly one {@link UsernameIDPrincipal}. Every
     *            {@link AttributePrincipal} of the Subject is mapped to a SAMLAttributeSatement. Every
     *            {@link RolePrincipal} is mapped to a special SAMLAttributeStatement that will be interpreted as roles
     *            during authorization.
     * @param pRoleAttributeName
     *            Specifies the name of the AttributeStatements used to store role information, e.g.
     *            <code>urn:conterra:names:sdi-suite:policy:attribute:role</code>
     * @param pIssuer
     *            The name that will appear as issuer of the SAMLAssertion.
     * @param pTarget
     *            The name of the consumer of the SAMLAssertion.
     * @param pTimeout
     *            The period of validity of the SAMLAssertion in seconds.
     * @param pAuthentMethod
     *            The authentication method that was used to identify the subject, e.g. <code>#password</code>. Go to
     *            the SAML specification part called <i>Assertions and Protocol for the OASIS Security Assertion Markup
     *            Language (SAML) V1.1</i> to find a list of authentication method IDs
     * @param pPrivKey
     *            The private key that will be used to sign the SAML Response.
     * @param pCert
     *            The certificate that belongs to the private key. It will be embedded into the SAMLResponse as part of
     *            the signature. If this is <code>null</code>, the ticket is not signed.
     * @param pBase64
     *            If <code>true</code> the SAML ticket will be Base64-encoded, it won't be encoded otherwise.
     * @return The ticket XML as a String.
     * @throws SAMLException
     * @throws IOException
     */
    public static String createSAMLResponseFromPrincipals(AuthenticationContext authContext,
            String pRoleAttributeName, String pIssuer, String pTarget, int pTimeout, String pAuthentMethod,
            Key pPrivKey, Certificate pCert, boolean pBase64) throws SAMLException, IOException {
        Subject subject = authContext.getSubject();
        // expect at least on userid principal instance
        UsernameIDPrincipal userId =
                (UsernameIDPrincipal) subject.getPrincipals(UsernameIDPrincipal.class).iterator().next();
        String lName = userId.getName();
        String lNameQualifier = userId.getScope().getName();

        SAMLAssertion assertion;
        SAMLStatement lAuthentStatement = null;
        SAMLSubject samlSubject;
        String subjectIP = null;
        String subjectDNS = null;
        java.util.Date authInstant = authContext.getAuthenticationTime();
        SAMLResponse response;

        Calendar cal = Calendar.getInstance();
        Calendar cal0 = (Calendar) cal.clone();
        cal.add(Calendar.SECOND, pTimeout);

        SAMLNameIdentifier identifier = new SAMLNameIdentifier(lName, lNameQualifier, "#ID");

        // Collection confMethods = new ArrayList(1);
        // confMethods.add("confirmationMethods");
        samlSubject = new SAMLSubject(identifier, null, null, null);

        Collection statements = new ArrayList();

        try {
            lAuthentStatement =
                    new SAMLAuthenticationStatement((SAMLSubject) samlSubject.clone(), pAuthentMethod, authInstant,
                            subjectIP, subjectDNS, null);

            statements.add(lAuthentStatement);

            Map attributeValuesMapping = getAttributesFromSubject(subject, pRoleAttributeName);
            Collection samlAtts = new ArrayList();
            Iterator lAttTableItr = attributeValuesMapping.entrySet().iterator();
            while (lAttTableItr.hasNext()) {
                Map.Entry tEntry = (Map.Entry) lAttTableItr.next();
                SAMLAttribute tAtt =
                        new SAMLAttribute((String) tEntry.getKey(), "def", null, 0, (Collection) tEntry.getValue());
                samlAtts.add(tAtt);
            }
            if (!samlAtts.isEmpty()) {
                SAMLAttributeStatement tAttStatement =
                        new SAMLAttributeStatement((SAMLSubject) samlSubject.clone(), samlAtts);
                statements.add(tAttStatement);
            }
        } catch (CloneNotSupportedException e) {
            sLogger.error("Error during SAMLAuthentication Statement creation.", e);
        }

        assertion = new SAMLAssertion(pIssuer, cal0.getTime(), cal.getTime(), null, null, statements);
        Collection assertions = new ArrayList(1);
        assertions.add(assertion);
        response = new SAMLResponse(null, pTarget, assertions, null);

        if (sLogger.isDebugEnabled()) {
            sLogger.debug("Generated SAMLResponse (yet unsigned):\n" + response);
        }

        Collection certList = new ArrayList(4);
        if (pCert != null) {
            certList.add(pCert);
            response.sign("http://www.w3.org/2000/09/xmldsig#rsa-sha1", pPrivKey, certList);
        }
        return pBase64 ? new String(response.toBase64()) : response.toString();

    }

    private static Map getAttributesFromSubject(final Subject subject, final String roleAttName) {
        Map attributes = new HashMap();
        Iterator attributeprincipals = subject.getPrincipals(AttributePrincipal.class).iterator();
        while (attributeprincipals.hasNext()) {
            AttributePrincipal attributePrincipal = (AttributePrincipal) attributeprincipals.next();
            String attName = attributePrincipal.getName();
            Collection attValues = (Collection) attributes.get(attName);
            if (attValues == null) {
                attValues = new LinkedList();
                attributes.put(attName, attValues);
            }
            // todo: how to support complex attribute mappings with a static method
            String attValue = attributePrincipal.getValue();
            attValues.add(attValue);
        }
        Iterator roleprincipals = subject.getPrincipals(RolePrincipal.class).iterator();
        Collection roles = new LinkedList();
        while (roleprincipals.hasNext()) {
            RolePrincipal rolePrincipal = (RolePrincipal) roleprincipals.next();
            // todo use scope ?
            roles.add(rolePrincipal.getName());
        }
        if (!roles.isEmpty()) {
            attributes.put(roleAttName, roles);
        }
        // todo explicite support for groups and/or other principals?
        return attributes;
    }

    // private static Set getPrincipals(SAMLResponse pSAMLResponse) throws
    // SAMLException {
    // HashSet lPrincipals = new HashSet(8);
    //
    // pSAMLResponse.verify();
    //
    // // TODO: check if response timed out
    //
    // SAMLAssertion samlFirstAssertion = (SAMLAssertion)
    // pSAMLResponse.getAssertions().next();
    //
    // // Iterate through statements and create appropriate principals
    // Iterator itrSamlStatements = samlFirstAssertion.getStatements();
    // while (itrSamlStatements.hasNext()) {
    // SAMLStatement tSamlStatement = (SAMLStatement) itrSamlStatements.next();
    //
    // // evaluate AuthenticationStatement
    // if (tSamlStatement instanceof SAMLAuthenticationStatement) {
    // SAMLAuthenticationStatement samlAuthnStatement =
    // (SAMLAuthenticationStatement) tSamlStatement;
    // SAMLSubject samlSubject = samlAuthnStatement.getSubject();
    // String userName = samlSubject.getName().getName();
    // String userID = samlSubject.getName().getNameQualifier();
    // lPrincipals.add(new UsernameIDPrincipal(userName, userID));
    // }
    //
    // // evaluate AttributeStatements
    // if (tSamlStatement instanceof SAMLAttributeStatement) {
    // SAMLAttributeStatement samlAttrStatement = (SAMLAttributeStatement)
    // tSamlStatement;
    //
    // Iterator itrSamlAttributes = samlAttrStatement.getAttributes();
    // while (itrSamlAttributes.hasNext()) {
    // SAMLAttribute tAtt = (SAMLAttribute) itrSamlAttributes.next();
    // Iterator itrSamlAttrValues = tAtt.getValues();
    // while (itrSamlAttrValues.hasNext()) {
    // String tValue = (String) itrSamlAttrValues.next();
    // lPrincipals.add(new AttributePrincipal(tAtt.getName(), tValue));
    // }
    // }
    // }
    // }
    // return lPrincipals;
    //
    // }

    // private static Set getPrincipals(Document pSAMLResponse) throws
    // SAMLException{
    // SAMLResponse lSAMLResponse = new
    // SAMLResponse(pSAMLResponse.getDocumentElement());
    // return getPrincipals(lSAMLResponse);
    // }

    // private static Set getPrincipals(String pSAMLResponse) throws
    // SAMLException {
    // SAMLResponse samlResponse = new SAMLResponse(new
    // ByteArrayInputStream(pSAMLResponse.getBytes()));
    // return getPrincipals(samlResponse);
    // }
}