/*******************************************************************************
 * Copyright (C) 2008
 * by 52 North Initiative for Geospatial Open Source Software GmbH
 *
 * Contact: Andreas Wytzisk
 * 52 North Initiative for Geospatial Open Source Software GmbH
 * Martin-Luther-King-Weg 24
 * 48155 Muenster, Germany
 * info@52north.org
 *
 * This program is free software; you can redistribute and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program (see gnu-gpl v2.txt). If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA or
 * visit the Free Software Foundation web page, http://www.fsf.org.
 ******************************************************************************/

package org.n52.security.authentication.loginmodule;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import javax.security.auth.Subject;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.security.auth.login.LoginException;
import javax.security.auth.spi.LoginModule;
import org.apache.log4j.Logger;
import org.n52.security.authentication.UsernamePasswordCredential;
import org.n52.security.authentication.callbacks.CredentialCallback;
import org.n52.security.authentication.principals.LoginNamePrincipal;
import org.n52.security.authentication.principals.RolePrincipal;
import org.n52.security.authentication.principals.UsernameIDPrincipal;
import org.n52.security.common.util.StringUtils;

/**
 * Logs in users by means of username/password, checked against an XML file containing username/password/role
 * information.
 * 
 * @author JanDrewnak
 * 
 */
public class FileLoginModule implements LoginModule {

    private static Map sUsers;

    private static final Logger LOG = Logger.getLogger(FileLoginModule.class);

    private Subject m_subject;

    private CallbackHandler m_callbackHandler;

    private Map m_sharedState;

    private Map m_options;

    private boolean m_debug = false;

    private Set m_principals;

    /*
     * (non-Javadoc)
     * 
     * @see javax.security.auth.spi.LoginModule#abort()
     */
    public boolean abort() throws LoginException {
        m_subject = null;
        m_callbackHandler = null;
        m_sharedState = null;
        m_options = null;
        m_debug = false;
        m_principals.clear();

        return true;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.security.auth.spi.LoginModule#commit()
     */
    public boolean commit() throws LoginException {
        m_subject.getPrincipals().addAll(m_principals);
        return true;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.security.auth.spi.LoginModule#login()
     */
    public boolean login() throws LoginException {

        boolean isBase64Encoded = Boolean.valueOf((String) m_options.get("credential.isBase64Encoded")).booleanValue();

        // get the initialization params
        String cacheUsersOption = (String) m_options.get("users.cache");
        boolean cacheUsers = Boolean.valueOf(cacheUsersOption).booleanValue();
        LOG.info("Setting user repository caching to: " + cacheUsers);

        String tUsersFilePath = (String) m_options.get("users.file.path");
        URL usersFileUrl;
        if (tUsersFilePath != null && !tUsersFilePath.equals("")) {
            try {
                usersFileUrl = new URL(tUsersFilePath);
                LOG.info("Loading user repository from " + usersFileUrl);
            } catch (MalformedURLException e) {
                LOG.error("Cannot load user file from " + tUsersFilePath);
                throw new LoginException("Internal service error. Please contact the service administrator.");
            }

        } else {
            usersFileUrl = this.getClass().getResource("/users.xml");
            LOG.info("Loading user repository from /users.xml of the classpath");
        }

        if (sUsers == null || !cacheUsers) {
            try {
                sUsers = new FileLoginUserManager(usersFileUrl).getUserMap();
            } catch (Exception e) {
                LOG.debug("Relative paths are appended to " + new File(".").getAbsolutePath());
                LOG.error("Could not load users from file " + usersFileUrl, e);
                throw new LoginException("Internal service error. Please contact the service administrator.");
            }
        }

        // get the credentials
        Set crendentialSet = m_subject.getPublicCredentials();
        Iterator it = crendentialSet.iterator();
        UsernamePasswordCredential credential = null;
        while (it.hasNext()) {
            Object publicCred = it.next();
            if (publicCred instanceof UsernamePasswordCredential) {
                credential = (UsernamePasswordCredential) it.next();
                break;
            }
        }
        if (credential == null) {
            // use callbackhandler
            CredentialCallback credentialCallback = new CredentialCallback(UsernamePasswordCredential.class);
            try {
                m_callbackHandler.handle(new Callback[] { credentialCallback });
            } catch (IOException e) {
                throw new LoginException("IOException occured " + e);
            } catch (UnsupportedCallbackException e) {
                throw new LoginException("CredentailCallback classes not supported :" + e);
            }
            credential = (UsernamePasswordCredential) credentialCallback.getCredential();
            if (credential == null) {
                throw new LoginException("no required credentials found");
            }
        }

        // check if credentials are given properly

        String user = credential.getUsername();
        String pass = new String(credential.getPassword());

        if (isBase64Encoded) {
            pass = StringUtils.decodeBase64(pass);
            user = StringUtils.decodeBase64(user);

        }

        FileLoginUser tUserToLogin = (FileLoginUser) sUsers.get(user);

        if (tUserToLogin == null) {
            LOG.debug("Authentication of " + user + " failed: username unknown");
            throw new LoginException("Unknown login name: " + user);
        }

        if (!tUserToLogin.getPassword().equals(pass)) {
            LOG.debug("Authentication of " + user + " failed: wrong password");
            throw new LoginException("Wrong password");
        }
        LOG.debug("Authentication of " + user + " sucessful");
        m_principals.add(new LoginNamePrincipal(tUserToLogin.getUsername()));
        m_principals.add(new UsernameIDPrincipal(tUserToLogin.getUsername()));
        String[] lUserToLoginRoles = tUserToLogin.getRoles();
        for (int i = 0, numRoles = lUserToLoginRoles.length; i < numRoles; i++) {
            m_principals.add(new RolePrincipal(lUserToLoginRoles[i]));
        }
        return true;

    }

    /*
     * (non-Javadoc) /* (non-Javadoc)
     * 
     * @see javax.security.auth.spi.LoginModule#logout()
     */
    public boolean logout() throws LoginException {
        Iterator it = m_subject.getPrincipals().iterator();

        while (it.hasNext()) {
            m_subject.getPrincipals().remove(it.next());
        }
        return true;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.security.auth.spi.LoginModule#initialize(javax.security.auth.Subject,
     * javax.security.auth.callback.CallbackHandler, java.util.Map, java.util.Map)
     */
    public void initialize(final Subject subject, final CallbackHandler callbackHandler, final Map sharedState,
            final Map options) {
        this.m_subject = subject;
        this.m_callbackHandler = callbackHandler;
        this.m_sharedState = sharedState;
        this.m_options = options;
        m_debug = "true".equalsIgnoreCase((String) options.get("debug"));
        this.m_principals = new HashSet(8);
    }
}
