/*******************************************************************************
 * Copyright (C) 2009
 * by 52 North Initiative for Geospatial Open Source Software GmbH
 * 
 * Contact: Andreas Wytzisk
 * 52 North Initiative for Geospatial Open Source Software GmbH
 * Martin-Luther-King-Weg 24
 * 48155 Muenster, Germany
 * info@52north.org
 * 
 * This program is free software; you can redistribute and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program (see gnu-gpl v2.txt). If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA or
 * visit the Free Software Foundation web page, http://www.fsf.org.
 ******************************************************************************/

package org.n52.security.authentication.loginmodule;

import java.io.Serializable;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.regex.Pattern;

/**
 * Class Options represents the configuration options of a login module.
 * <p/>
 * This class is introduced to encapsulate the access to common options
 * and therefor to minimize programming errors.
 * <br/>
 * This class is created in the initialize method of
 * {@link org.n52.security.authentication.loginmodule.AbstractLoginModule}
 * and provided via a getter method to sub classes.
 *
 * @author Marko Reiprecht
 *         Created on 15.09.2007
 * @version $Revision: $
 */
public class Options implements Serializable {

    // ------------------------------ FIELDS ------------------------------

    /**
     * Field OPTION_TRY_FIRST_PASS (true|false).
     * <p/>
     * If true, the first LoginModule in the stack saves the password entered,
     * and subsequent LoginModules also try to use it. If authentication fails,
     * the LoginModules prompt for a new password and retry the authentication.
     *
     * @see <a href="http://java.sun.com/j2se/1.4.2/docs/guide/security/jaas/JAASLMDevGuide.html#initialize">
     *      Description of the initialize method in the LoginModule Developer's Guide
     *      </a>
     */
    public static final String OPTION_TRY_FIRST_PASS = "try_first_pass";
    /**
     * Field OPTION_USE_FIRST_PASS (true|false).
     * <p/>
     * If true, the first LoginModule in the stack saves the password entered,
     * and subsequent LoginModules also try to use it.
     * LoginModules do not prompt for a new password if authentication fails (authentication simply fails).
     *
     * @see <a href="http://java.sun.com/j2se/1.4.2/docs/guide/security/jaas/JAASLMDevGuide.html#initialize">
     *      Description of the initialize method in the LoginModule Developer's Guide
     *      </a>
     */
    public static final String OPTION_USE_FIRST_PASS = "use_first_pass";
    /**
     * Field OPTION_TRY_MAPPED_PASS (true|false).
     * <p/>
     * If true, the first LoginModule in the stack saves the password entered,
     * and subsequent LoginModules attempt to map it into their service-specific password.
     * If authentication fails, the LoginModules prompt for a new password and retry the authentication.
     *
     * @see <a href="http://java.sun.com/j2se/1.4.2/docs/guide/security/jaas/JAASLMDevGuide.html#initialize">
     *      Description of the initialize method in the LoginModule Developer's Guide
     *      </a>
     */
    public static final String OPTION_TRY_MAPPED_PASS = "try_mapped_pass";
    /**
     * Field OPTION_USE_MAPPED_PASS (true|false).
     * <p/>
     * If true, the first LoginModule in the stack saves the password entered,
     * and subsequent LoginModules attempt to map it into their service-specific password.
     * LoginModules do not prompt for a new password if authentication fails (authentication simply fails).
     *
     * @see <a href="http://java.sun.com/j2se/1.4.2/docs/guide/security/jaas/JAASLMDevGuide.html#initialize">
     *      Description of the initialize method in the LoginModule Developer's Guide
     *      </a>
     */
    public static final String OPTION_USE_MAPPED_PASS = "use_mapped_pass";
    /**
     * Field OPTION_MODULEBANNER (true|false).
     * <p/>
     * If true, then when invoking the CallbackHandler,
     * the LoginModule provides a TextOutputCallback as the first Callback,
     * which describes the LoginModule performing the authentication.
     *
     * @see <a href="http://java.sun.com/j2se/1.4.2/docs/guide/security/jaas/JAASLMDevGuide.html#initialize">
     *      Description of the initialize method in the LoginModule Developer's Guide
     *      </a>
     */
    public static final String OPTION_MODULEBANNER = "moduleBanner";

    /**
     * Field OPTION_DISABLED (true|false).
     * <p/>
     * If true, instructs a LoginModule to ignore every call to login.
     */
    public static final String OPTION_DISABLED = "disabled";

    /**
     * The serialVersionUID.
     *
     * @see java.io.Serializable
     */
    private static final long serialVersionUID = 2451329698634194233L;
    /**
     * Field IS_BOOLEAN_PATTERN contains the Regex for accepted boolean string representations.
     */
    private static final Pattern IS_BOOLEAN_PATTERN = Pattern.compile("^(?i:true|yes|1)$");


    /**
     * Field m_options thats the wrapped module options.
     */
    private Map m_options;

    // --------------------------- CONSTRUCTORS ---------------------------

    /**
     * Constructor Options creates a new Options instance.
     *
     * @param state the ${FIELD_NAME}.
     */
    public Options(final Map state) {
        assert state != null;
        m_options = state;
    }

    // -------------------------- OTHER METHODS --------------------------

    /**
     * Method getAsDouble gets the option value for the key as Double.
     *
     * @param key          the key.
     * @param defaultvalue the defaultvalue.
     * @return double.
     * @throws NumberFormatException if error occurs.
     */
    public double getAsDouble(final String key, final double defaultvalue)
            throws NumberFormatException {
        Object o = get(key);
        return o == null ? defaultvalue
                         : o instanceof Double ? ((Double) o).doubleValue()
                                               : Double.parseDouble(String.valueOf(o));
    }

    /**
     * Method getAsFloat gets the option value for the key as Float.
     *
     * @param key          the key.
     * @param defaultvalue the defaultvalue.
     * @return float.
     * @throws NumberFormatException if error occurs.
     */
    public float getAsFloat(final String key, final float defaultvalue)
            throws NumberFormatException {
        Object o = get(key);
        return o == null ? defaultvalue
                         : o instanceof Float ? ((Float) o).floatValue()
                                              : Float.parseFloat(String.valueOf(o));
    }

    /**
     * Method getAsInt gets the option value for the key as Integer.
     *
     * @param key          the key.
     * @param defaultvalue the defaultvalue.
     * @return int.
     * @throws NumberFormatException if the set value is no Integer.
     */
    public int getAsInt(final String key, final int defaultvalue)
            throws NumberFormatException {
        Object o = get(key);
        return o == null ? defaultvalue
                         : o instanceof Integer ? ((Integer) o).intValue()
                                                : Integer.parseInt(String.valueOf(o));
    }

    /**
     * Method getAsLong gets the option value for the key as Long.
     *
     * @param key          the key.
     * @param defaultvalue the defaultvalue.
     * @return long.
     * @throws NumberFormatException if the set value is no Long.
     */
    public long getAsLong(final String key, final long defaultvalue) {
        Object o = get(key);
        return o == null ? defaultvalue
                         : o instanceof Long ? ((Long) o).longValue()
                                             : Long.parseLong(String.valueOf(o));
    }

    /**
     * Method getAsString gets the option value for the key as a String.
     *
     * @param key          the key.
     * @param defaultvalue defines the default value if the option is empty.
     * @return String.
     */
    public String getAsString(final String key, final String defaultvalue) {
        Object o = get(key);
        return o == null ? defaultvalue : String.valueOf(o);
    }

    /**
     * Method getAsStringArray gets the option value for the key as String[].
     *
     * @param key          the key.
     * @param defaultvalue the defaultvalue.
     * @param delemiter    the delemiter like in StringTokenizer.
     * @return String[].
     * @see java.util.StringTokenizer
     */
    public String[] getAsStringArray(final String key,
                                     final String[] defaultvalue,
                                     final String delemiter) {
        Object o = get(key);
        return o == null ? defaultvalue
                         : o instanceof String[] ? (String[]) o
                                                 : split(String.valueOf(o), delemiter);
    }

    /**
     * Method splits the given string into pieces indicated by the delemiter chars.
     *
     * @param tosplit   the tosplit.
     * @param delemiter the delemiter.
     * @return String[].
     * @see java.util.StringTokenizer
     */
    private String[] split(final String tosplit, final String delemiter) {
        // here no use of the split method to avoid regex problems.
        StringTokenizer stringTokenizer = new StringTokenizer(tosplit, delemiter, false);
        String[] splittedstring = new String[stringTokenizer.countTokens()];
        for (int i = 0; stringTokenizer.hasMoreTokens(); ++i) {
            splittedstring[i] = stringTokenizer.nextToken();
        }
        return splittedstring;
    }

    /**
     * Method is gets the option value for the key as boolean.
     *
     * @param key         the key.
     * @param defaulvalue the default value.
     * @return boolean the default value.
     */
    public boolean is(final String key, final boolean defaulvalue) {
        return contains(key) ? is(key) : defaulvalue;
    }

    /**
     * Method checks if the state information contains the key.
     *
     * @param key of type String identifying a state information.
     * @return boolean true if key in state.
     */
    public boolean contains(final String key) {
        return m_options.containsKey(key);
    }

    /**
     * Method checks if the disabled option is set.
     *
     * @return boolean.
     * @see #OPTION_DISABLED
     */
    public boolean isDisabled() {
        return is(OPTION_DISABLED);
    }

    /**
     * Method gets the boolean value from the option stored under the given key.
     * If the option value is null it returns false,
     * if it is a {@link Boolean} it returns its value,
     * otherwise it tries to get a Boolean value from the String representation of the value.
     *
     * @param key the option key.
     * @return boolean.
     */
    public boolean is(final String key) {
        Object option = get(key);
        if (option == null) {
            return false;
        }
        if (option instanceof Boolean) {
            return ((Boolean) option).booleanValue();
        }
        return IS_BOOLEAN_PATTERN.matcher(String.valueOf(option)).matches();
    }

    /**
     * Method gets the state information for the key.
     *
     * @param key of type String identifying a state information.
     * @return Object or null if (!contains(key))
     */
    public Object get(final String key) {
        return m_options.get(key);
    }

    /**
     * Methods checks for an empty shared state.
     *
     * @return boolean true if no state information is available.
     */
    public boolean isEmpty() {
        return m_options.isEmpty();
    }

    /**
     * Method checks if the moduleBanner option is set.
     *
     * @return boolean.
     * @see #OPTION_MODULEBANNER
     */
    public boolean isModuleBanner() {
        return is(OPTION_MODULEBANNER);
    }

    /**
     * Method checks if the tryFirstPass option is set.
     *
     * @return boolean.
     * @see #OPTION_TRY_FIRST_PASS
     */
    public boolean isTryFirstPass() {
        return is(OPTION_TRY_FIRST_PASS);
    }

    /**
     * Method checks if the tryMappedPass option is set.
     *
     * @return boolean.
     * @see #OPTION_TRY_MAPPED_PASS
     */
    public boolean isTryMappedPass() {
        return is(OPTION_TRY_MAPPED_PASS);
    }

    /**
     * Method checks if the useFirstPass option is set.
     *
     * @return boolean.
     * @see #OPTION_USE_FIRST_PASS
     */
    public boolean isUseFirstPass() {
        return is(OPTION_USE_FIRST_PASS);
    }

    /**
     * Method checks if the useMappedPass option is set.
     *
     * @return boolean.
     * @see #OPTION_USE_MAPPED_PASS
     */
    public boolean isUseMappedPass() {
        return is(OPTION_USE_MAPPED_PASS);
    }

    /**
     * Method getAs gets the option value for the key as type of the given class.
     *
     * @param key          the key.
     * @param expectedType the expected class of the key.
     * @return Object of type expectedType.
     * @throws NumberFormatException if error occurs.
     */
    public Object getAs(final String key, final Class expectedType)
            throws IllegalArgumentException {
        Object o = get(key);
        if (o != null && expectedType.isInstance(o)) {
            return o;
        }
        throw new IllegalArgumentException(
                "object <" + o + "> stored under key <" + key + "> is not of type <" + expectedType.getName() + ">");
    }
}
