/*******************************************************************************
 * Copyright (C) 2009
 * by 52 North Initiative for Geospatial Open Source Software GmbH
 * 
 * Contact: Andreas Wytzisk
 * 52 North Initiative for Geospatial Open Source Software GmbH
 * Martin-Luther-King-Weg 24
 * 48155 Muenster, Germany
 * info@52north.org
 * 
 * This program is free software; you can redistribute and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program (see gnu-gpl v2.txt). If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA or
 * visit the Free Software Foundation web page, http://www.fsf.org.
 ******************************************************************************/

package org.n52.security.authentication.principals;

import org.n52.security.common.attributes.Attribute;
import org.n52.security.common.attributes.AttributeValue;
import org.n52.security.common.attributes.StringAttributeValue;

/**
 * Container for simple parameter/value principals.
 */
public class AttributePrincipal extends NamePrincipal {

    // ------------------------------ FIELDS ------------------------------

    private static final long serialVersionUID = -7096929626934197914L;

    private Attribute m_attribute;

    // --------------------------- CONSTRUCTORS ---------------------------

    /**
     * Constructor creates a new instance.
     *
     * @param attribute the attribute.
     * @throws IllegalArgumentException if error occurs.
     */
    public AttributePrincipal(final Attribute attribute)
            throws IllegalArgumentException {
        this(attribute, Scope.GLOBAL);
    }

    /**
     * Constructor creates a new instance.
     *
     * @param attribute the attribute.
     * @param scope     the scope.
     * @throws IllegalArgumentException if error occurs.
     */
    public AttributePrincipal(final Attribute attribute, final Scope scope)
            throws IllegalArgumentException {
        super(attribute.getKey(), scope);
        m_attribute = attribute;
    }

    /**
     * Constructor creates a new instance.
     *
     * @param key   the key.
     * @param value the value.
     * @throws IllegalArgumentException if error occurs.
     */
    public AttributePrincipal(final String key, final String value)
            throws IllegalArgumentException {
        this(key, value, Scope.GLOBAL);
    }

    /**
     * Constructor creates a new instance.
     *
     * @param key   the key.
     * @param value the value.
     * @param scope the scope.
     * @throws IllegalArgumentException if error occurs.
     */
    public AttributePrincipal(final String key, final String value, final Scope scope)
            throws IllegalArgumentException {
        super(key, scope);
        m_attribute = new Attribute(key, new StringAttributeValue(value));
    }

    // --------------------- GETTER / SETTER METHODS ---------------------

    /**
     * Method gets the attribute.
     *
     * @return Attribute.
     */
    public Attribute getAttribute() {
        return m_attribute;
    }

    /**
     * Method gets the attribute value.
     * short cut for getAttribute().getValue()
     *
     * @return AttributeValue.
     */
    public AttributeValue getAttributeValue() {
        return m_attribute.getValue();
    }

    /**
     * @return the string value of this attribute
     */
    public String getValue() {
        Object unspecifiedval = getAttribute().getValue().getUnspecifiedValue();
        return String.valueOf(unspecifiedval == null ? "" : unspecifiedval);
    }

    /**
     * Returns the equaltiy.
     *
     * @param o the other object.
     * @return boolean.
     */
    public boolean equals(final Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof AttributePrincipal)) {
            return false;
        }
        if (!super.equals(o)) {
            return false;
        }

        final AttributePrincipal that = (AttributePrincipal) o;

        if (!m_attribute.equals(that.m_attribute)) {
            return false;
        }

        return true;
    }

    /**
     * Returns the hashCode.
     *
     * @return int.
     */
    public int hashCode() {
        int result = super.hashCode();
        result = 31 * result + m_attribute.hashCode();
        return result;
    }
}