/*******************************************************************************
 * Copyright (C) 2009
 * by 52 North Initiative for Geospatial Open Source Software GmbH
 * 
 * Contact: Andreas Wytzisk
 * 52 North Initiative for Geospatial Open Source Software GmbH
 * Martin-Luther-King-Weg 24
 * 48155 Muenster, Germany
 * info@52north.org
 * 
 * This program is free software; you can redistribute and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program (see gnu-gpl v2.txt). If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA or
 * visit the Free Software Foundation web page, http://www.fsf.org.
 ******************************************************************************/

package org.n52.security.common.attributes;

import java.io.Serializable;

/**
 * This Class represents an Key-Value-Attribute.
 * It also implements Serializable, but to be it, its value
 * has also to be serializable.
 *
 * @author Marko Reiprecht
 * @version $Revision: 1748 $
 */
public class Attribute implements Serializable {

    // ------------------------------ FIELDS ------------------------------

    /**
     * Field serialVersionUID.
     */
    private static final long serialVersionUID = 481658090077657175L;

    /**
     * Field m_key holds the key or name of the attribute.
     */
    protected String m_key;
    /**
     * Field m_value holds thee value of this attribute.
     *
     * @see org.n52.security.common.attributes.AttributeValue}
     */
    protected AttributeValue m_value;

    // --------------------------- CONSTRUCTORS ---------------------------

    /**
     * Creates an Attribute.
     * Both parameters are adopted to be not null.
     * For Attributes with a value of 'null' use {@link AttributeValue#EMPTY_VALUE}.
     *
     * @param key   the key or name of the attribute
     * @param value the value of the attribute.
     */
    public Attribute(final String key, final AttributeValue value) {
        assert key != null;
        assert value != null;

        this.m_key = key;
        this.m_value = value;
    }

    // --------------------- GETTER / SETTER METHODS ---------------------

    /**
     * @return Returns the key.
     */
    public String getKey() {
        return m_key;
    }

    /**
     * @return Returns the value.
     */
    public AttributeValue getValue() {
        return m_value;
    }

    // ------------------------ CANONICAL METHODS ------------------------

    /**
     * Tests for equality.
     * An attribute equals an other attribute if the key and the value are equal.
     *
     * @param o the test object.
     * @return true if key and the value equals o's value and key.
     */
    public boolean equals(final Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }

        final Attribute attribute = (Attribute) o;

        if (!m_key.equals(attribute.m_key)) {
            return false;
        }
        if (!m_value.equals(attribute.m_value)) {
            return false;
        }

        return true;
    }

    /**
     * Generates a hashcode based of the hashcode of the key and value.
     *
     * @return a hashcode.
     */
    public int hashCode() {
        int result;
        result = m_key.hashCode();
        result = 31 * result + m_value.hashCode();
        return result;
    }

    /**
     * Generates an informal string.
     * This method should only used for logging purposes.
     *
     * @return a string representation.
     */
    public String toString() {
        final StringBuffer sb = new StringBuffer();
        sb.append("org.n52.security.common.attributes.Attribute");
        sb.append("{key='").append(getKey()).append('\'');
        sb.append(", value=").append(getValue());
        sb.append('}');
        return sb.toString();
    }
}