/*******************************************************************************
 * Copyright (C) 2008
 * by 52 North Initiative for Geospatial Open Source Software GmbH
 *
 * Contact: Andreas Wytzisk
 * 52 North Initiative for Geospatial Open Source Software GmbH
 * Martin-Luther-King-Weg 24
 * 48155 Muenster, Germany
 * info@52north.org
 *
 * This program is free software; you can redistribute and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program (see gnu-gpl v2.txt). If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA or
 * visit the Free Software Foundation web page, http://www.fsf.org.
 *
 * Author: Marko Reiprecht
 * Created: 30.05.2008
 ******************************************************************************/
/* $Id: $ */
package org.n52.security.common.xml;

import java.util.LinkedList;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.traversal.NodeFilter;

/**
 * Class DocumentTraverser is able to traverse a dom document and calls a DocumentVisitor for each traversed node.
 *
 * @author Marko Reiprecht
 *         Created on 30.05.2008
 * @version $Revision: $
 */
public class DocumentTraverser {

    /**
     * Method traverses the dom tree starting at node using depth first traversal,
     * it skips the start node if visitStartNode==false.
     *
     * @param node           the startNode.
     * @param visitor        the visitor.
     * @param visitStartNode the visitStartNode.
     */
    public void traverseDepthFirst(final Node node, final DocumentVisitor visitor, final boolean visitStartNode) {
        short currentAcceptDecision = 0;
        if (visitStartNode) {
            traverseDepthFirst(node, visitor);
        } else {
            // save iteration over the childs
            // this makes it possible to remove or append childs
            // on the fly.
            Node child = node.getFirstChild();
            while (child != null) {
                // if child is removed during visiting we must store the next sibling
                Node beforNext = child.getNextSibling();
                // let visitor do it's work
                traverseDepthFirst(child, visitor);
                // now check if new childs appended
                Node afterNext = child.getNextSibling();
                child = afterNext == null ? beforNext : afterNext;
            }
        }
    }

    /**
     * Method traverses the dom tree starting at node using depth first traversal (childs first).
     *
     * @param node    the startNode.
     * @param visitor the visitor.
     */
    public void traverseDepthFirst(final Node node, final DocumentVisitor visitor) {
        short currentAcceptDecision = visitor.acceptNode(node);
        switch (currentAcceptDecision) {
            case NodeFilter.FILTER_ACCEPT:
                visitor.preVisit(node);
                visitor.visit(node);
                break;
            case NodeFilter.FILTER_REJECT:
                return;
                // cancel method
            case NodeFilter.FILTER_SKIP:
                break;
        }
        // if node is attributes, no further processing (otherwise the attribute text node is visited, too)
        if (node.getNodeType() != Node.ATTRIBUTE_NODE) {
            // before going in the childs traverse the attributes
            NamedNodeMap attributes = node.getAttributes();
            if (attributes != null) {
                for (int i = 0, l = attributes.getLength(); i < l; ++i) {
                    Node attr = attributes.item(i);
                    traverseDepthFirst(attr, visitor);
                }
            }
            // save iteration over the childs
            // this makes it possible to remove or append childs
            // on the fly.
            Node child = node.getFirstChild();
            while (child != null) {
                // if child is removed during visiting we must store the next sibling
                Node beforNext = child.getNextSibling();
                // let visitor do it's work
                traverseDepthFirst(child, visitor);
                // now check if new childs appended
                Node afterNext = child.getNextSibling();
                child = afterNext == null ? beforNext : afterNext;
            }
        }
        if (NodeFilter.FILTER_ACCEPT == currentAcceptDecision) {
            visitor.postVisit(node);
        }
    }

    /**
     * Method traverses the node in breadth first order (silblings first).
     *
     * @param node           the node.
     * @param visitor        the visitor.
     * @param visitStartNode the visitStartNode.
     */
    public void traverseBreadthFirst(final Node node, final DocumentVisitor visitor, final boolean visitStartNode) {
        if (visitStartNode) {
            traverseBreadthFirst(node, visitor);
        } else {
            // this is a non recursive traversal !
            LinkedList queue = new LinkedList();
            NodeList childs = node.getChildNodes();
            for (int i = 0, l = childs.getLength(); i < l; ++i) {
                Node child = childs.item(i);
                queue.addLast(child);
            }
            traverseBreadthFirst(visitor, queue);
        }
    }

    /**
     * Method traverses the node in breadth first order (silblings first).
     *
     * @param node    the node.
     * @param visitor the visitor.
     */
    public void traverseBreadthFirst(final Node node, final DocumentVisitor visitor) {
        // this is a non recursive traversal !
        LinkedList queue = new LinkedList();
        queue.addFirst(node);
        traverseBreadthFirst(visitor, queue);
    }

    /**
     * Method traverses the node in the queue in breadth first order (silblings first)
     * and calls the DocumentVisitor.visit method.
     *
     * @param visitor   the visitor.
     * @param nodeQueue the nodeQueue.
     */
    private void traverseBreadthFirst(final DocumentVisitor visitor, final LinkedList nodeQueue) {
        // this is a non recursive traversal !
        while (!nodeQueue.isEmpty()) {
            Node current = (Node) nodeQueue.removeFirst();
            short acceptDecision = visitor.acceptNode(current);
            if (acceptDecision == NodeFilter.FILTER_ACCEPT) {
                visitor.visit(current);
            }
            if (acceptDecision != NodeFilter.FILTER_REJECT
                && current.getNodeType() != Node.ATTRIBUTE_NODE) {
                // before going in the childs add the attributes
                NamedNodeMap attributes = current.getAttributes();
                if (attributes != null) {
                    for (int i = 0, l = attributes.getLength(); i < l; ++i) {
                        Node attr = attributes.item(i);
                        nodeQueue.addFirst(attr);
                    }
                }
                NodeList childs = current.getChildNodes();
                for (int i = 0, l = childs.getLength(); i < l; ++i) {
                    Node child = childs.item(i);
                    nodeQueue.addLast(child);
                }
            }
        }
    }
}
