/*******************************************************************************
 * Copyright (C) 2008
 * by 52 North Initiative for Geospatial Open Source Software GmbH
 *
 * Contact: Andreas Wytzisk
 * 52 North Initiative for Geospatial Open Source Software GmbH
 * Martin-Luther-King-Weg 24
 * 48155 Muenster, Germany
 * info@52north.org
 *
 * This program is free software; you can redistribute and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program (see gnu-gpl v2.txt). If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA or
 * visit the Free Software Foundation web page, http://www.fsf.org.
 *
 * Author: Marko Reiprecht
 * Created: 30.05.2008
 ******************************************************************************/
/* $Id: $ */
package org.n52.security.common.xml;

import org.w3c.dom.Attr;
import org.w3c.dom.CDATASection;
import org.w3c.dom.Comment;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.DocumentType;
import org.w3c.dom.Element;
import org.w3c.dom.Entity;
import org.w3c.dom.EntityReference;
import org.w3c.dom.Node;
import org.w3c.dom.Notation;
import org.w3c.dom.ProcessingInstruction;
import org.w3c.dom.Text;
import org.w3c.dom.traversal.NodeFilter;

/**
 * Class DocumentVisitorAdapter implements
 * a DocumentVisitor and delegates the untyped visit(Node) methods to typed methods, like visit(Element).
 * <p/>
 * Thats a support class, which makes it very easy to implement visitors reacting on special node types.
 *
 * @author Marko Reiprecht
 *         Created on 30.05.2008
 * @version $Revision: $
 */
public class DocumentVisitorAdapter implements DocumentVisitor {

    // ------------------------ INTERFACE METHODS ------------------------


    // --------------------- Interface DocumentVisitor ---------------------

    /**
     * @see DocumentVisitor#visit(Node)
     */
    public void visit(final Node node) {
        switch (node.getNodeType()) {
            case Node.ATTRIBUTE_NODE:
                visit((Attr) node);
                break;
            case Node.CDATA_SECTION_NODE:
                visit((CDATASection) node);
                break;
            case Node.COMMENT_NODE:
                visit((Comment) node);
                break;
            case Node.DOCUMENT_FRAGMENT_NODE:
                visit((DocumentFragment) node);
                break;
            case Node.DOCUMENT_NODE:
                visit((Document) node);
                break;
            case Node.DOCUMENT_TYPE_NODE:
                visit((DocumentType) node);
                break;
            case Node.ELEMENT_NODE:
                visit((Element) node);
                break;
            case Node.ENTITY_NODE:
                visit((Entity) node);
                break;
            case Node.ENTITY_REFERENCE_NODE:
                visit((EntityReference) node);
                break;
            case Node.NOTATION_NODE:
                visit((Notation) node);
                break;
            case Node.PROCESSING_INSTRUCTION_NODE:
                visit((ProcessingInstruction) node);
                break;
            case Node.TEXT_NODE:
                visit((Text) node);
                break;
        }
    }

    /**
     * @see DocumentVisitor#preVisit(Node)
     */
    public void preVisit(final Node node) {
        switch (node.getNodeType()) {
            case Node.ATTRIBUTE_NODE:
                preVisit((Attr) node);
                break;
            case Node.CDATA_SECTION_NODE:
                preVisit((CDATASection) node);
                break;
            case Node.COMMENT_NODE:
                preVisit((Comment) node);
                break;
            case Node.DOCUMENT_FRAGMENT_NODE:
                preVisit((DocumentFragment) node);
                break;
            case Node.DOCUMENT_NODE:
                preVisit((Document) node);
                break;
            case Node.DOCUMENT_TYPE_NODE:
                preVisit((DocumentType) node);
                break;
            case Node.ELEMENT_NODE:
                preVisit((Element) node);
                break;
            case Node.ENTITY_NODE:
                preVisit((Entity) node);
                break;
            case Node.ENTITY_REFERENCE_NODE:
                preVisit((EntityReference) node);
                break;
            case Node.NOTATION_NODE:
                preVisit((Notation) node);
                break;
            case Node.PROCESSING_INSTRUCTION_NODE:
                preVisit((ProcessingInstruction) node);
                break;
            case Node.TEXT_NODE:
                preVisit((Text) node);
                break;
        }
    }

    /**
     * @see DocumentVisitor#postVisit(Node)
     */
    public void postVisit(final Node node) {
        switch (node.getNodeType()) {
            case Node.ATTRIBUTE_NODE:
                postVisit((Attr) node);
                break;
            case Node.CDATA_SECTION_NODE:
                postVisit((CDATASection) node);
                break;
            case Node.COMMENT_NODE:
                postVisit((Comment) node);
                break;
            case Node.DOCUMENT_FRAGMENT_NODE:
                postVisit((DocumentFragment) node);
                break;
            case Node.DOCUMENT_NODE:
                postVisit((Document) node);
                break;
            case Node.DOCUMENT_TYPE_NODE:
                postVisit((DocumentType) node);
                break;
            case Node.ELEMENT_NODE:
                postVisit((Element) node);
                break;
            case Node.ENTITY_NODE:
                postVisit((Entity) node);
                break;
            case Node.ENTITY_REFERENCE_NODE:
                postVisit((EntityReference) node);
                break;
            case Node.NOTATION_NODE:
                postVisit((Notation) node);
                break;
            case Node.PROCESSING_INSTRUCTION_NODE:
                postVisit((ProcessingInstruction) node);
                break;
            case Node.TEXT_NODE:
                postVisit((Text) node);
                break;
        }
    }

    // --------------------- Interface NodeFilter ---------------------


    /**
     * Method accepts all node types.
     *
     * @param n the n.
     * @return short.
     */
    public short acceptNode(final Node n) {
        return NodeFilter.FILTER_ACCEPT;
    }

    // -------------------------- OTHER METHODS --------------------------

    /**
     * Method postVisit.
     *
     * @param element the element.
     */
    public void postVisit(final Element element) {}

    /**
     * Method postVisit.
     *
     * @param attr the attr.
     */
    public void postVisit(final Attr attr) {}

    /**
     * Method postVisit.
     *
     * @param cdata the cdata.
     */
    public void postVisit(final CDATASection cdata) {}

    /**
     * Method postVisit.
     *
     * @param text the text.
     */
    public void postVisit(final Text text) {}

    /**
     * Method postVisit.
     *
     * @param comment the comment.
     */
    public void postVisit(final Comment comment) {}

    /**
     * Method postVisit.
     *
     * @param docType the docType.
     */
    public void postVisit(final DocumentType docType) {}

    /**
     * Method postVisit.
     *
     * @param document the document.
     */
    public void postVisit(final Document document) {}

    /**
     * Method postVisit.
     *
     * @param docFrag the docFrag.
     */
    public void postVisit(final DocumentFragment docFrag) {}

    /**
     * Method postVisit.
     *
     * @param entity the entity.
     */
    public void postVisit(final Entity entity) {}

    /**
     * Method postVisit.
     *
     * @param entityRef the entityRef.
     */
    public void postVisit(final EntityReference entityRef) {}

    /**
     * Method postVisit.
     *
     * @param processingInstruction the processingInstruction.
     */
    public void postVisit(final ProcessingInstruction processingInstruction) {}

    /**
     * Method postVisit.
     *
     * @param notation the notation.
     */
    public void postVisit(final Notation notation) {}

    /**
     * Method preVisit.
     *
     * @param attr the attr.
     */
    public void preVisit(final Attr attr) {}

    /**
     * Method preVisit.
     *
     * @param cdata the cdata.
     */
    public void preVisit(final CDATASection cdata) {}

    /**
     * Method preVisit.
     *
     * @param text the text.
     */
    public void preVisit(final Text text) {}

    /**
     * Method preVisit.
     *
     * @param comment the comment.
     */
    public void preVisit(final Comment comment) {}

    /**
     * Method preVisit.
     *
     * @param docType the docType.
     */
    public void preVisit(final DocumentType docType) {}

    /**
     * Method preVisit.
     *
     * @param document the document.
     */
    public void preVisit(final Document document) {}

    /**
     * Method preVisit.
     *
     * @param docFrag the docFrag.
     */
    public void preVisit(final DocumentFragment docFrag) {}

    /**
     * Method preVisit.
     *
     * @param element the element.
     */
    public void preVisit(final Element element) {}

    /**
     * Method preVisit.
     *
     * @param entity the entity.
     */
    public void preVisit(final Entity entity) {}

    /**
     * Method preVisit.
     *
     * @param entityRef the entityRef.
     */
    public void preVisit(final EntityReference entityRef) {}

    /**
     * Method preVisit.
     *
     * @param processingInstruction the processingInstruction.
     */
    public void preVisit(final ProcessingInstruction processingInstruction) {}

    /**
     * Method preVisit.
     *
     * @param notation the notation.
     */
    public void preVisit(final Notation notation) {}

    /**
     * Method visit.
     *
     * @param attr the attr.
     */
    public void visit(final Attr attr) {}

    /**
     * Method visit.
     *
     * @param cdata the cdata.
     */
    public void visit(final CDATASection cdata) {}

    /**
     * Method visit.
     *
     * @param text the text.
     */
    public void visit(final Text text) {}

    /**
     * Method visit.
     *
     * @param comment the comment.
     */
    public void visit(final Comment comment) {}

    /**
     * Method visit.
     *
     * @param docType the docType.
     */
    public void visit(final DocumentType docType) {}

    /**
     * Method visit.
     *
     * @param document the document.
     */
    public void visit(final Document document) {}

    /**
     * Method visit.
     *
     * @param docFrag the docFrag.
     */
    public void visit(final DocumentFragment docFrag) {}

    /**
     * Method visit.
     *
     * @param element the element.
     */
    public void visit(final Element element) {}

    /**
     * Method visit.
     *
     * @param entity the entity.
     */
    public void visit(final Entity entity) {}

    /**
     * Method visit.
     *
     * @param entityRef the entityRef.
     */
    public void visit(final EntityReference entityRef) {}

    /**
     * Method visit.
     *
     * @param processingInstruction the processingInstruction.
     */
    public void visit(final ProcessingInstruction processingInstruction) {}

    /**
     * Method visit.
     *
     * @param notation the notation.
     */
    public void visit(final Notation notation) {}
}