/*******************************************************************************
 * Copyright (C) 2008
 * by 52 North Initiative for Geospatial Open Source Software GmbH
 *
 * Contact: Andreas Wytzisk
 * 52 North Initiative for Geospatial Open Source Software GmbH
 * Martin-Luther-King-Weg 24
 * 48155 Muenster, Germany
 * info@52north.org
 *
 * This program is free software; you can redistribute and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program (see gnu-gpl v2.txt). If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA or
 * visit the Free Software Foundation web page, http://www.fsf.org.
 *
 * Author: Marko Reiprecht
 * Created: 15.07.2008
 * Modified: $Date $
 ******************************************************************************/
/* $Id: $ */
package org.n52.security.common.xml;

import java.util.HashMap;
import java.util.Map;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.commons.pool.PoolableObjectFactory;
import org.apache.commons.pool.impl.GenericObjectPool;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

/**
 * Class XMLPathCtx encapsulates context informations for xpath expressions.
 * <p/>
 * Use this class to create new XMLPath objectes and combinations of them.
 * <p/>
 * This class wraps the javax.xml.xpath package with performance improvments and simpler useage.
 *
 * @author Marko Reiprecht
 *         Created on 15.07.2008
 * @version $Revision: $
 */
public class XMLPathCtx {

    // ------------------------------ FIELDS ------------------------------

    /**
     * The Logger.
     */
    private static final Log LOG = LogFactory.getLog(XMLPathCtx.class);

    private static XPathPool xPathPool = new XPathPool(XPathFactory.newInstance());

    private XPathPool m_xPathPool;
    private Map m_nameToPreCompiledExpressionMap = new HashMap();
    private SimpleNamespaceContext m_namespaceContext = new SimpleNamespaceContext();

    // --------------------------- CONSTRUCTORS ---------------------------

    /**
     * Constructor creates a new instance.
     *
     * @param xPathPool the xPathPool.
     */
    protected XMLPathCtx(final XPathPool xPathPool) {
        m_xPathPool = xPathPool;
    }

    // --------------------- GETTER / SETTER METHODS ---------------------

    /**
     * Method gets the XPathPool.
     *
     * @return XPathPool.
     */
    private XPathPool getXPathPool() {
        return m_xPathPool;
    }

    // -------------------------- OTHER METHODS --------------------------

    /**
     * Method adds prefix to namespace mapping, so the prefix can be used to qualify xpath expressions.
     *
     * @param prefix    the prefix.
     * @param namespace the namespace.
     * @return XMLPathCtx.
     */
    public XMLPathCtx addNamespace(final String prefix, final String namespace) {
        m_namespaceContext.add(prefix, namespace);
        return this;
    }

    /**
     * Method creates a new XMLPathCtx instance.
     * The instance uses the internaly xpath instance pool.
     *
     * @return XMLPathCtx.
     */
    public static XMLPathCtx createNew() {
        return new XMLPathCtx(xPathPool);
    }

    /**
     * Method findIn constructs a new XMLPath starting in a Document source.
     *
     * @param doc the doc.
     * @return XMLPath.
     */
    public XMLPath findIn(final Document doc) {
        return new XMLPath.SourceXMLPath(this, doc);
    }

    /**
     * Method findIn constructs a new XMLPath starting in a Node source.
     *
     * @param node the node.
     * @return XMLPath.
     */
    public XMLPath findIn(final Node node) {
        return new XMLPath.SourceXMLPath(this, node);
    }

    /**
     * Method findIn constructs a new XMLPath starting in a NodeList source.
     *
     * @param nodeList the nodeList.
     * @return XMLPath.
     */
    public XMLPath findIn(final NodeList nodeList) {
        return new XMLPath.SourceXMLPath(this, nodeList);
    }

    /**
     * Method findIn constructs a new XMLPath starting in a InputSource source.
     *
     * @param source the source.
     * @return XMLPath.
     */
    public XMLPath findIn(final InputSource source) {
        return new XMLPath.SourceXMLPath(this, source);
    }

    /**
     * Method findIn constructs a new XMLPath starting in a string source.
     *
     * @param source the source.
     * @return XMLPath.
     */
    public XMLPath findIn(final String source) {
        return new XMLPath.SourceXMLPath(this, source);
    }

    /**
     * Method looks up an XPathExpression.
     * A XPathExpression can be precompiled by using the register mehtods.
     *
     * @param xpath the xpath.
     * @return XPathExpression.
     */
    protected XPathExpression lookUp(final String xpath) {
        // first simple native implementation
        XPathExpression expression = (XPathExpression) m_nameToPreCompiledExpressionMap.get(xpath);
        return expression == null ? preCompile(xpath) : expression;
    }

    /**
     * Method pre compiles an xpath expression.
     *
     * @param xpath the xpath.
     * @return XPathExpression.
     */
    private XPathExpression preCompile(final String xpath) {
        XPath path = getXPathPool().acquire();
        try {
            path.setNamespaceContext(m_namespaceContext);
            return path.compile(xpath);
        } catch (XPathExpressionException e) {
            throw new XMLPathException(e);
        } finally {
            getXPathPool().release(path);
        }
    }

    /**
     * Method registers an xpath as precompiled XPathExpression.
     *
     * @param xpath the xpath.
     * @return XMLPathCtx.
     */
    public XMLPathCtx register(final String xpath) {
        return register(xpath, xpath);
    }

    /**
     * Method registers an xpath as precompiled XPathExpression with a special name.
     * <p/>
     * After that you can use the name when ever an xpath is needed.
     *
     * @param name  the name.
     * @param xpath the xpath.
     * @return XMLPathCtx.
     */
    public XMLPathCtx register(final String name, final String xpath) {
        m_nameToPreCompiledExpressionMap.put(name, preCompile(xpath));
        return this;
    }

    // -------------------------- INNER CLASSES --------------------------

    /**
     * Class XPathPool.
     *
     * @author Marko Reiprecht
     *         Created on 30.05.2008
     * @version $Revision: $
     */
    private static class XPathPool {

        private GenericObjectPool m_xpathPool;

        /**
         * Constructor creates a new instance.
         *
         * @param factory the XPathFactory to use.
         */
        private XPathPool(final XPathFactory factory) {
            GenericObjectPool.Config config = new GenericObjectPool.Config();
            // max number of idle builders per option
            config.maxIdle = 5;
            // max number of active builder per option
            config.maxActive = 20;
            // don't grow, fail, if the pool is exhausted.
            // Blocking can lead to bad error states, so this poll will fail,
            // if more than 20 parsers are parallel in use.
            config.whenExhaustedAction = GenericObjectPool.WHEN_EXHAUSTED_GROW;
            // but only wait for maximal one second befor throw an exception
            // wait is obsolete if exhausted fail is configured.
            //config.maxWait = 1000;
            m_xpathPool = new GenericObjectPool(new XPathPoolableObjectFactory(factory), config);
        }

        /**
         * Method acquires a new XPath from the pool.
         *
         * @return XPath.
         */
        public XPath acquire() {
            try {
                return (XPath) m_xpathPool.borrowObject();
            } catch (Exception e) {
                throw new DOMSerializerException("XPath can not retrieved from pool", e);
            }
        }

        /**
         * Method releases a XPath back in the pool.
         *
         * @param xPath the transformer.
         */
        public void release(final XPath xPath) {
            try {
                m_xpathPool.returnObject(xPath);
            } catch (Exception e) {
                if (LOG.isWarnEnabled()) { LOG.warn("XPath could not returned in pool"); }
            }
        }
    }

    /**
     * Class TransformerKeyedPoolableObjectFactory.
     *
     * @author Marko Reiprecht
     *         Created on 30.05.2008
     * @version $Revision: $
     */
    private static class XPathPoolableObjectFactory implements PoolableObjectFactory {

        private XPathFactory m_xPathFactory;

        /**
         * Constructor creates a new instance.
         *
         * @param xPathFactory the xPathFactory.
         */
        private XPathPoolableObjectFactory(final XPathFactory xPathFactory) {
            m_xPathFactory = xPathFactory;
        }

        /**
         * Method makes a XPath.
         *
         * @return XPath.
         * @throws Exception if error occurs.
         */
        public Object makeObject() throws Exception {
            return m_xPathFactory.newXPath();
        }

        /**
         * Method destroys the XPath.
         *
         * @param xpath the XPath.
         * @throws Exception if error occurs.
         */
        public void destroyObject(final Object xpath) throws Exception {
            // nothing
        }

        /**
         * Method validates the XPath.
         *
         * @param xpath the XPath.
         * @return boolean.
         */
        public boolean validateObject(final Object xpath) {
            return true;
        }

        /**
         * Method activates the XPath.
         *
         * @param xpath the XPath.
         * @throws Exception if error occurs.
         */
        public void activateObject(final Object xpath) throws Exception {
            // nothing
        }

        /**
         * Method passivates the XPath.
         *
         * @param xpath the XPath.
         * @throws Exception if error occurs.
         */
        public void passivateObject(final Object xpath) throws Exception {
            ((XPath) xpath).reset();
        }
    }
}
